﻿namespace App287
{
  using System;
  using System.Numerics;
  using Windows.UI.Composition;
  using Windows.UI.Xaml.Hosting;

  class VisualRectangleDrawer : RectangleDrawer
  {
    protected override void OnDrawCanvasChanging()
    {
      base.OnDrawCanvasChanging();

      if (base.DrawCanvas != null)
      {
        ElementCompositionPreview.SetElementChildVisual(base.DrawCanvas, null);
      }
      this.containerVisual?.Dispose();
      this.containerVisual = null;

      this.rotationAnimation?.Dispose();
      this.rotationAnimation = null;

      this.brush?.Dispose();
      this.brush = null;

      this.compositor = null;

    }
    protected override void OnDrawCanvasChanged()
    {
      if (base.DrawCanvas != null)
      {
        var canvasVisual = ElementCompositionPreview.GetElementVisual(
          base.DrawCanvas);

        this.compositor = canvasVisual.Compositor;

        this.containerVisual = this.compositor.CreateContainerVisual();

        this.brush = this.compositor.CreateColorBrush(base.Color);

        // NB: I wanted to animate the colour here but, to date, I do not seem to be
        // able to get colour animations to work in the composition bits and I can't
        // find anything on the web that uses them
#if COLOUR_ANIMATIONS
        this.colorAnimation = this.compositor.CreateColorKeyFrameAnimation();
        this.colorAnimation.InsertKeyFrame(0.0f, Color.FromArgb(0x55, 0x00, 0x00, 0x00));
        this.colorAnimation.InsertKeyFrame(1.0f, Color.FromArgb(0x55, 0xFF, 0xFF, 0xFF));
        this.colorAnimation.Duration = TimeSpan.FromSeconds(1);
        this.colorAnimation.IterationBehavior = AnimationIterationBehavior.Forever;

        this.brush.StartAnimation("Color", this.colorAnimation);

#endif // COLOUR_ANIMATIONS

        this.rotationAnimation = this.compositor.CreateScalarKeyFrameAnimation();
        this.rotationAnimation.InsertKeyFrame(0.0f, 0.0f);
        this.rotationAnimation.InsertKeyFrame(1.0f, 360.0f);
        this.rotationAnimation.Duration = TimeSpan.FromSeconds(1);
        this.rotationAnimation.IterationBehavior = AnimationIterationBehavior.Forever;

        this.opacityAnimation = this.compositor.CreateScalarKeyFrameAnimation();
        this.opacityAnimation.InsertKeyFrame(0.0f, 0.0f);
        this.opacityAnimation.InsertKeyFrame(1.0f, 1.0f);
        this.opacityAnimation.Duration = TimeSpan.FromSeconds(1);
        this.opacityAnimation.IterationBehavior = AnimationIterationBehavior.Forever;

        ElementCompositionPreview.SetElementChildVisual(this.DrawCanvas,
          this.containerVisual);
      }
    }
    public override void ClearRectangles()
    {
      this.containerVisual?.Children.RemoveAll();
    }
    public override void DrawRectangle(int x, int y, int width, int height)
    {
      var rectangleVisual = this.compositor.CreateSpriteVisual();
      rectangleVisual.Offset = new Vector3(x, y, 0);
      rectangleVisual.Size = new Vector2(width, height);
      rectangleVisual.CenterPoint = new Vector3((width / 2), (height / 2), 0);
      rectangleVisual.Brush = this.brush;
      rectangleVisual.StartAnimation("RotationAngleInDegrees", this.rotationAnimation);
      rectangleVisual.StartAnimation("Opacity", this.opacityAnimation);
      containerVisual.Children.InsertAtBottom(rectangleVisual);
    }
#if COLOUR_ANIMATIONS
    ColorKeyFrameAnimation colorAnimation;
#endif // COLOUR_ANIMATIONS
    ScalarKeyFrameAnimation opacityAnimation;
    ScalarKeyFrameAnimation rotationAnimation;
    Compositor compositor;
    ContainerVisual containerVisual;
    CompositionColorBrush brush;
  }
}